/*
 * Et netera, http://boss.etnetera.cz - Copyright (C) 2012 
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License (version 2.1) as published by the Free Software
 * Foundation.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * See the GNU Lesser General Public License for more details:
 * http://www.gnu.org/licenses/lgpl-3.0.txt
 * 
 */
package cz.wicketstuff.boss.flow.processor.basic;

import java.io.Serializable;
import java.util.HashMap;
import java.util.Map;

import cz.wicketstuff.boss.flow.model.IFlowState;
import cz.wicketstuff.boss.flow.processor.IFlowStateDataFactory;
import cz.wicketstuff.boss.flow.processor.NoSuchStateException;
import cz.wicketstuff.boss.flow.processor.StateDataException;

public class DefaultFlowStateDataFactory implements IFlowStateDataFactory, Serializable {

	private static final long serialVersionUID = 1L;
	
	private Map<IFlowState, Class<Serializable>> stateDataMap;	
	private boolean exceptionIfStateNotExist;
	
	public DefaultFlowStateDataFactory() {
		this(null);
	}

	public DefaultFlowStateDataFactory(
			Map<IFlowState, Class<Serializable>> stateDataMap) {
		this(stateDataMap, false);
	}

	public DefaultFlowStateDataFactory(
			Map<IFlowState, Class<Serializable>> stateDataMap,
			boolean exceptionIfStateNotExist) {
		this.stateDataMap = stateDataMap;
		this.exceptionIfStateNotExist = exceptionIfStateNotExist;
	}

	@Override
	public Serializable createStateData(IFlowState flowState) throws NoSuchStateException, StateDataException {
		Class<Serializable> c = getStateDataMap().get(flowState);
		if(c == null) {
			if(exceptionIfStateNotExist) {
				throw new NoSuchStateException("Cannot find state '" + flowState.getStateName() + "'");
			}
			return null;
		}		
		try {
			return c.newInstance();
		} catch (InstantiationException e) {
			throw new StateDataException("Cannot instantiate class for '" + c.getName() + "' for state '" + flowState.getStateName() + "', probably the constructor thrown an exception", e);
		} catch (IllegalAccessException e) {
			throw new StateDataException("Cannot instantiate class for '" + c.getName() + "' for state '" + flowState.getStateName() + "', probably constructor is private or an interface is mapped instead a regular class.", e);
		}
	}

	public boolean isExceptionIfStateNotExist() {
		return exceptionIfStateNotExist;
	}

	public void setExceptionIfStateNotExist(boolean exceptionIfStateNotExist) {
		this.exceptionIfStateNotExist = exceptionIfStateNotExist;
	}

	public Map<IFlowState, Class<Serializable>> getStateDataMap() {
		if(stateDataMap == null) {
			stateDataMap = new HashMap<IFlowState, Class<Serializable>>();
		}
		return stateDataMap;
	}

	public void setStateDataMap(Map<IFlowState, Class<Serializable>> stateDataMap) {
		this.stateDataMap = stateDataMap;
	}
	
	@Override
	protected void finalize() throws Throwable {
		if(stateDataMap != null) {
			stateDataMap = null;
		}
		super.finalize();
	}


}
